<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://themehigh.com
 * @since      1.0.0
 *
 * @package    woocommerce-email-customizer-pro
 * @subpackage woocommerce-email-customizer-pro/public
 */
if(!defined('WPINC')){	die; }

if(!class_exists('THWEC_Public')):
 
class THWEC_Public {
	/**
	 * Name of the plugin
	 *
	 * @access   private
	 * @var      $plugin_name    The string used to uniquely identify this plugin.
	 */
	private $plugin_name;

	/**
	 * Version of the plugin
	 *
	 * @access   private
	 * @var      $version    The string used to identify version of this plugin.
	 */
	private $version;
	
	/**
	 * Manages the current language set in WPML
	 *
	 * @access   private
	 * @var      $current_lang    WPML current language
	 */
	private $current_lang = null;
	
	/**
	 * Manages the WPML active status
	 *
	 * @access   private
	 * @var      $wpml_active    WPML active or not
	 */
	private $wpml_active = false;
	
	/**
	 * Manages the default language set in WPML 
	 *
	 * @access   private
	 * @var      $wpml_default_lang    WPML default language
	 */
	private $wpml_default_lang = '';
    
    /**
	 * Manages the subjects for all supported email notifications
	 *
	 * @access   private
	 * @var      $subjects    array of subjects
	 */
    public $subjects;
    
    /**
	 * Manages the email customizer settings
	 *
	 * @access   private
	 * @var      $settings    array of settings for customizer
	 */
    private $settings = array();
    
    /**
	 * Manages the data required for order status manager
	 *
	 * @access   private
	 * @var      $wc_status_manager    order status manager helper
	 */
    private $wc_status_manager = array();

    /**
	 * List of emails
	 *
	 * @access   public
	 * @var      $email_list    emails
	 */
    public $email_list = array();

    /**
	 * WC_Emails class instance
	 *
	 * @access   public
	 * @var      $woo_mailer  class instance
	 */
    public $woo_mailer = null;

    /**
	 * Manages the list of WooCommerce default admin emails
	 *
	 * @access   public
	 * @var      $woo_mailer  admin email list
	 */
    public $admin_emails = array();

    /**
	 * Construct
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		add_action('after_setup_theme', array($this, 'define_public_hooks'));
	}

	/**
	 * Enqueue styles and scripts
	 *
	 */	
	public function enqueue_styles_and_scripts() {
		global $wp_scripts;
		
		if(is_product()){
			$debug_mode = apply_filters('thwec_debug_mode', false);
			$suffix = $debug_mode ? '' : '.min';
			$jquery_version = isset($wp_scripts->registered['jquery-ui-core']->ver) ? $wp_scripts->registered['jquery-ui-core']->ver : '1.9.2';
		}
	}
	
	/**
	 * Define functionalities for the public end
	 *
	 */	
	public function define_public_hooks(){
		$this->settings = THWEC_Utils::get_template_settings();
		$this->template_map = THWEC_Utils::get_template_map( $this->settings );
		$wc_hook_priority = apply_filters('wc_get_template_priority', 10);
		add_filter('woocommerce_email_styles', array($this, 'th_woocommerce_email_styles') );
		add_filter( 'wc_get_template', array( $this, 'get_woo_template'), $wc_hook_priority, 5 );
		// add_filter( 'woocommerce_email_attachments', array($this, 'thwec_woo_attachments'), 10, 4 );
	}

	/**
	 * Locate template file for the email notification
	 *
	 * @param  string $template template
	 * @param  array $template_name template name
	 * @param  string $template_path path of template
	 * @return string template file
	 */	
	
	public function get_woo_template( $template, $template_name, $args, $template_path, $default_path ){
		if( isset( $_GET['page'] ) && $_GET['page'] == sanitize_text_field( 'th_email_customizer_pro') && isset( $_GET['preview'] ) ){
			return $template;
		}
		if($template_name === "emails/email-order-details.php" || $template_name === "emails/email-downloads.php"){
			return $template;
		}
		$template_id = isset($args['email']) && isset($args['email']->id) && !empty($args['email']->id) ? $args['email']->id : false;
		$integrations = apply_filters('thwec_locate_integration_template', false, $template_id, $template_name, $this->template_map);
		$located_template = $this->get_email_template_path($integrations);
		if( $located_template ){
			return $located_template;
		}
		return $template;
	}

	/**
	 * Template path for the email notification
	 *
	 * @param  string $t_name template name
	 * @return string template path found or boolean if not
	 */	
	public function get_email_template_path($t_name){
    	$tpath = false;
    	$email_template_path = THWEC_CUSTOM_TEMPLATE_PATH.$t_name.'.php';
    	if(file_exists($email_template_path)){
    		// $this->manage_woocommerce_hooks(true);
    	   	$tpath = $email_template_path;
    	}
    	return $tpath;
    }

    /**
	 * Additional styles for email templates
	 *
	 * @param  string $buffer default email styles passed from WooCommerce
	 * @return string email styles
	 */	
	public function th_woocommerce_email_styles($buffer){
		$css = '';
		$styles = $this->wecmf_template_css_compatibility(); 
		$styles .= THWEC_Utils::wecm_email_styles();
		$styles.= apply_filters('thwec_woo_css_override', $css);
		return $buffer.$styles;
	}

	/**
	 * Free version compatible styles
	 *
	 * Free version templates that are not edited in the premium version (but assigned to email) atleast once, require free version styles.
	 *
	 * @return string styles
	 */	
	public function wecmf_template_css_compatibility(){
		$styles = '#tpf_t_builder #template_container,#tpf_t_builder #template_header,#tpf_t_builder #template_body,#tpf_t_builder #template_footer{width:100% !important;}';
		$styles.= '#tpf_t_builder #template_container{width:100% !important;border:0px none transparent !important;}';
		$styles .= '#tpf_t_builder #body_content > table:first-child > tbody > tr > td{padding:15px 0px !important;}'; //To remove the padding after header when woocommerce header hook used in template (48px 48px 0px) 
		$styles.= '#tpf_t_builder #wrapper{padding:0;background-color:transparent;}';
		$styles.= '#tpf_t_builder .thwec-block-text-holder a{color: #1155cc !important;}';
		$styles.= '#tpf_t_builder .thwecmf-columns p{color:#636363;font-size:14px;}';
		$styles.= '#tpf_t_builder .thwecmf-columns .td .td{padding:12px;}';
		$styles.= '#tpf_t_builder .thwecmf-columns .address{font-size:14px;}';
		return $styles;
	}
}
endif;